# Generated by Django 5.0.6 on 2024-07-03 14:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("currency", "0001_initial"),
        ("geography", "0001_initial"),
        ("io", "0007_alter_exportsource_query_params"),
        ("tags", "0001_initial"),
        ("wbfdm", "0021_delete_instrumentdailystatistics"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrument",
            name="cusip",
            field=models.CharField(blank=True, help_text="CUSIP", max_length=255, null=True, verbose_name="CUSIP"),
        ),
        migrations.AddField(
            model_name="option",
            name="open_interest_20d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (20D)", null=True, verbose_name="Open Interest 20D"
            ),
        ),
        migrations.AddField(
            model_name="option",
            name="open_interest_50d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (50D)", null=True, verbose_name="Open Interest 50D"
            ),
        ),
        migrations.AddField(
            model_name="option",
            name="open_interest_5d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (5D)", null=True, verbose_name="Open Interest 5D"
            ),
        ),
        migrations.AddField(
            model_name="option",
            name="volatility",
            field=models.FloatField(blank=True, null=True, verbose_name="Volatility"),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="open_interest_20d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (20D)", null=True, verbose_name="Open Interest 20D"
            ),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="open_interest_50d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (50D)", null=True, verbose_name="Open Interest 50D"
            ),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="open_interest_5d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (5D)", null=True, verbose_name="Open Interest 5D"
            ),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="volatility",
            field=models.FloatField(blank=True, null=True, verbose_name="Volatility"),
        ),
        migrations.AlterField(
            model_name="instrument",
            name="sedol",
            field=models.CharField(
                blank=True,
                help_text="Stock Exchange Daily Official List",
                max_length=255,
                null=True,
                verbose_name="SEDOL",
            ),
        ),
        migrations.AlterField(
            model_name="instrument",
            name="valoren",
            field=models.CharField(
                blank=True, help_text="Valoren Number", max_length=255, null=True, verbose_name="Valoren Number"
            ),
        ),
    ]
