# Generated by Django 5.0.9 on 2024-12-19 12:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0027_remove_instrument_unique_ric_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrumentprice",
            name="annualized_daily_volatility",
            field=models.FloatField(blank=True, null=True, verbose_name="Annualized Volatility"),
        ),
    ]
