# Generated by Django 4.2.11 on 2024-04-10 14:46

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0017_instrument_instrument_security_index"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="instrument",
            index=models.Index(fields=["level"], name="instrument_level_index"),
        ),
        migrations.AddIndex(
            model_name="instrument",
            index=models.Index(fields=["parent"], name="instrument_parent_index"),
        ),
    ]
