# Generated by Django 5.0.6 on 2024-05-17 13:07

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0020_optionaggregate_option_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="InstrumentDailyStatistics",
        ),
    ]
